<?php
/**
 * This file is part of the wp-forecast plugin for WordPress
 *
 * Copyright 2006-2015  Hans Matzen  (email : webmaster at tuxlog dot de)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @package wp-forecast
 */

if ( ! function_exists( 'accu_xml_parser' ) ) {
	global $wpf_weather, $wpf_pstack, $wpf_go_ahead, $wpf_fc_daynumber;

	/**
	 * Build the url from the parameters and fetch the weather-data.
	 * Rreturn it as one long string.
	 *
	 * @param string $uri the accuweather URI.
	 * @param string $loc the accuweather lcoation code.
	 * @param string $metric which units to use.
	 */
	function accu_get_weather( $uri, $loc, $metric ) {
		$url = $uri . 'location=' . urlencode( $loc ) . '&metric=' . $metric;
		$xml = fetch_url( $url );
		return $xml;
	}

	/**
	 * Start_element() - called for every start tag.
	 *
	 * @param object $parser     the xml parser object.
	 * @param string $name       name of the starttag.
	 * @param string $attribute  name of the attribute.
	 */
	function accu_start_element( $parser, $name, $attribute ) {
		global $wpf_pstack, $wpf_go_ahead, $wpf_fc_daynumber, $wpf_weather;
		$wpf_path_table = array(
			'/ADC_DATABASE/UNITS/TEMP'                     => 'un_temp',
			'/ADC_DATABASE/UNITS/DIST'                     => 'un_dist',
			'/ADC_DATABASE/UNITS/SPEED'                    => 'un_speed',
			'/ADC_DATABASE/UNITS/PRES'                     => 'un_pres',
			'/ADC_DATABASE/UNITS/PREC'                     => 'un_prec',
			'/ADC_DATABASE/LOCAL/CITY'                     => 'city',
			'/ADC_DATABASE/LOCAL/STATE'                    => 'state',
			'/ADC_DATABASE/LOCAL/TIME'                     => 'time',
			'/ADC_DATABASE/LOCAL/LAT'                      => 'lat',
			'/ADC_DATABASE/LOCAL/LON'                      => 'lon',
			'/ADC_DATABASE/LOCAL/GMTDIFF'                  => 'gmtdiff',
			'/ADC_DATABASE/CURRENTCONDITIONS/PRESSURE'     => 'pressure',
			'/ADC_DATABASE/CURRENTCONDITIONS/TEMPERATURE'  => 'temperature',
			'/ADC_DATABASE/CURRENTCONDITIONS/REALFEEL'     => 'realfeel',
			'/ADC_DATABASE/CURRENTCONDITIONS/HUMIDITY'     => 'humidity',
			'/ADC_DATABASE/CURRENTCONDITIONS/WEATHERTEXT'  => 'weathertext',
			'/ADC_DATABASE/CURRENTCONDITIONS/WEATHERICON'  => 'weathericon',
			'/ADC_DATABASE/CURRENTCONDITIONS/WINDSPEED'    => 'windspeed',
			'/ADC_DATABASE/CURRENTCONDITIONS/WINDDIRECTION' => 'winddirection',
			'/ADC_DATABASE/CURRENTCONDITIONS/WINDGUSTS'    => 'wgusts',
			'/ADC_DATABASE/CURRENTCONDITIONS/UVINDEX'      => 'uvindex',
			'/ADC_DATABASE/PLANETS/SUN'                    => 'sun',
			'/ADC_DATABASE/FORECAST/DAY/OBSDATE'           => 'fc_obsdate',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/TXTSHORT'  => 'fc_dt_short',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/WEATHERICON' => 'fc_dt_icon',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/HIGHTEMPERATURE' => 'fc_dt_htemp',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/LOWTEMPERATURE' => 'fc_dt_ltemp',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/WINDSPEED' => 'fc_dt_windspeed',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/WINDDIRECTION' => 'fc_dt_winddir',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/WEATHERICON' => 'fc_nt_icon',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/HIGHTEMPERATURE' => 'fc_nt_htemp',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/LOWTEMPERATURE' => 'fc_nt_ltemp',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/WINDSPEED' => 'fc_nt_windspeed',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/WINDDIRECTION' => 'fc_nt_winddir',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/WINDGUST'  => 'fc_dt_wgusts',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/WINDGUST' => 'fc_nt_wgusts',
			'/ADC_DATABASE/FORECAST/DAY/DAYTIME/MAXUV'     => 'fc_dt_maxuv',
			'/ADC_DATABASE/FORECAST/DAY/NIGHTTIME/MAXUV'   => 'fc_nt_maxuv',
			'/ADC_DATABASE/FAILURE'                        => 'failure',
		);
		$wpf_pstack    .= "/$name";

		if ( isset( $wpf_path_table[ $wpf_pstack ] ) ) {
			$wpf_go_ahead = $wpf_path_table[ $wpf_pstack ];

			if ( '0' != $wpf_fc_daynumber && '' != $wpf_fc_daynumber ) {
				$wpf_go_ahead = $wpf_go_ahead . '_' . $wpf_fc_daynumber;
			}
		}
		// for forecast days.

		if ( '/ADC_DATABASE/FORECAST/DAY' == $wpf_pstack ) {
			$wpf_fc_daynumber = $attribute['NUMBER'];
		}
		// for current sun rise/set.

		if ( '/ADC_DATABASE/PLANETS/SUN' == $wpf_pstack ) {
			$wpf_weather['sun'] = $attribute['RISE'] . ' ' . $attribute['SET'];
		}
		// for current time daylightsavings.

		if ( '/ADC_DATABASE/LOCAL/GMTDIFF' == $wpf_pstack ) {

			if ( '' != $attribute['DAYLIGHTSAVINGS'] ) {
				$wpf_weather['gmtdiffdls'] = $attribute['DAYLIGHTSAVINGS'];
			} else {
				$wpf_weather['gmtdiffdls'] = 0;
			}
		}
		// for current uvindex.

		if ( '/ADC_DATABASE/CURRENTCONDITIONS/UVINDEX' == $wpf_pstack ) {
			$wpf_weather['uvindex'] = $attribute['INDEX'];
		}
	}

	/**
	 * End_element() - called for every end tag.
	 *
	 * @param object $parser the XML parser object.
	 * @param string $name name of the xml tag.
	 */
	function accu_end_element( $parser, $name ) {
		global $wpf_pstack, $wpf_fc_daynumber;
		// reduce xml path.
		$wpf_pstack = substr( $wpf_pstack, 0, strrpos( $wpf_pstack, '/' ) );

		if ( 'DAY' == $name ) {
			$wpf_fc_daynumber = 0;
		}
	}

	/**
	 * Daten() - called for everey cdata.
	 *
	 * @param object $parser the XML parser object.
	 * @param strin  $data   cdata from the xml tag.
	 */
	function accu_daten( $parser, $data ) {
		global $wpf_weather, $wpf_go_ahead;

		if ( strlen( $wpf_go_ahead ) > 0 && 'sun' != $wpf_go_ahead && 'gmtdiffdls' != $wpf_go_ahead && 'uvindex' != $wpf_go_ahead ) {
			$wpf_weather[ $wpf_go_ahead ] = $data;
			$wpf_go_ahead                 = '';
		}
	}

	/**
	 * Parses the xml for the paths in path_tabelle.
	 *
	 * @param string $xmlstring the complete xml string.
	 */
	function accu_xml_parser( $xmlstring ) {
		global $wpf_weather, $wpf_pstack, $wpf_go_ahead, $wpf_fc_daynumber;
		$wpf_weather      = array();
		$xmlerror         = '';
		$wpf_pstack       = '';
		$wpf_go_ahead     = '';
		$wpf_fc_daynumber = '0';
		// create an xml parser object.
		$parser = xml_parser_create();
		// set parameters for xml parser.
		xml_parser_set_option( $parser, XML_OPTION_CASE_FOLDING, true );
		// set handler for start and end-tags.
		xml_set_element_handler( $parser, 'accu_start_element', 'accu_end_element' );
		// set handler for CDATA.
		xml_set_character_data_handler( $parser, 'accu_daten' );
		// try to parse the xml.

		if ( ! xml_parse( $parser, $xmlstring, true ) ) {
			// Error --> stop execution.
			$xmlerror = 'XML Fehler: ' . xml_error_string( xml_get_error_code( $parser ) ) . ' in Zeile ' . xml_get_current_line_number( $parser );
		}
		// Vom XML-Parser belegten Speicher freigeben.
		xml_parser_free( $parser );
		// check for error.

		if ( '' != $xmlerror ) {
			$wpf_weather = array();
		}
		// and return result, empty array if error.
		return $wpf_weather;
	}

	/**
	 * Parse xml and extract locations as an array.
	 * for later use in the admin form.
	 *
	 * @param string $xml the complete xml string.
	 */
	function accu_get_locations( $xml ) {
		global $loc, $wpf_i;
		$wpf_i = 0;
		$loc   = array();
		// start_element() - wird vom XML-Parser bei öffnenden Tags aufgerufen.

		if ( ! function_exists( 's_element' ) ) {
			/**
			 * S_element() - called for every start tag.
			 *
			 * @param object $parser     the xml parser object.
			 * @param string $name       name of the starttag.
			 * @param string $attribute  name of the attribute.
			 */
			function s_element( $parser, $name, $attribute ) {
				global $loc, $wpf_i;

				if ( 'LOCATION' == $name ) {
					$loc[ $wpf_i ]             = array();
					$loc[ $wpf_i ]['city']     = $attribute['CITY'];
					$loc[ $wpf_i ]['state']    = $attribute['STATE'];
					$loc[ $wpf_i ]['location'] = $attribute['LOCATION'];
					$wpf_i++;
				}
			}
		}
		// end_element() - dummy function.

		if ( ! function_exists( 'e_element' ) ) {
			/**
			 * E_element() - called for every end tag.
			 *
			 * @param object $parser the XML parser object.
			 * @param string $name name of the xml tag.
			 */
			function e_element( $parser, $name ) {
			}
		}
		// Instanz des XML-Parsers erzeugen.
		$parser = xml_parser_create();
		// Parameter des XML-Parsers setzen.
		xml_parser_set_option( $parser, XML_OPTION_CASE_FOLDING, true );
		// Handler für Elemente ( öffnende / schließende Tags ) setzen.
		xml_set_element_handler( $parser, 's_element', 'e_element' );
		// try to parse the xml.

		if ( ! xml_parse( $parser, $xml, true ) ) {
			// Fehler -> Ausführung abbrechen.
			die( 'XML Fehler: ' . esc_attr( xml_error_string( xml_get_error_code( $parser ) ) ) . ' in Zeile ' . esc_attr( xml_get_current_line_number( $parser ) ) );
		}
		// Vom XML-Parser belegten Speicher freigeben.
		xml_parser_free( $parser );
	}

	/**
	 * Returns the widget data as an array one line per item.
	 *
	 * @param string $wpfcid            the widget id.
	 * @param string $language_override the lanugage iso code.
	 */
	function accu_forecast_data( $wpfcid = 'A', $language_override = null ) {
		$wpf_vars = get_wpf_opts( $wpfcid );

		if ( ! empty( $language_override ) ) {
			$wpf_vars['wpf_language'] = $language_override;
		}

		$w = maybe_unserialize( wpf_get_option( 'wp-forecast-cache' . $wpfcid ) );

		// get translations.
		global $wpf_lang_dict;
		include WP_PLUGIN_DIR . '/wp-forecast/widgetlang/wp-forecast-' . strtolower( str_replace( '_', '-', $wpf_vars['wpf_language'] ) ) . '.php';
		$wpf_lang_dict[ $wpf_vars['wpf_language'] ] = $wpf_lang;

		$weather_arr = array();

		// calc values for current conditions.
		if ( ! isset( $w['failure'] ) ) {
			// determine link to accuweather service for location.
			if ( is_numeric( $wpf_vars['location'] ) ) {
				$weather_arr['servicelink'] = 'http://www.accuweather.com/quick-look.aspx?partner=accuweather&amp;loc=' . $wpf_vars['locname'] . '&amp;metric=' . $wpf_vars['metric'];
			} else {
				$weather_arr['servicelink'] = 'http://www.accuweather.com/quick-look.aspx?partner=accuweather&amp;loc=' . $wpf_vars['location'] . '&amp;metric=' . $wpf_vars['metric'];
			}
			// next line is for compatibility.
			$weather_arr['acculink'] = $weather_arr['servicelink'];
			$weather_arr['location'] = $wpf_vars['locname'];
			$weather_arr['locname']  = $w['city'] . ' ' . $w['state'];
			$lt                      = time() - gmdate( 'Z' ); // this is the GMT.
			$ct                      = $lt + ( 3600 * ( $w['gmtdiff'] ) ); // local time.

			if ( 1 == $w['gmtdiffdls'] ) {
				$ct += 3600; // time with daylightsavings.
			}
			$ct                      = $ct + $wpf_vars['timeoffset'] * 60; // add or subtract time offset.
			$weather_arr['blogdate'] = date_i18n( $wpf_vars['fc_date_format'], $ct );
			$weather_arr['blogtime'] = date_i18n( $wpf_vars['fc_time_format'], $ct );
			$cts                     = $w['fc_obsdate_1'] . ' ' . $w['time'];
			$ct                      = strtotime( $cts );
			// $ct = $ct + $wpf_vars['timeoffset'] * 60; // add or subtract time offset.
			$weather_arr['accudate']    = date_i18n( $wpf_vars['fc_date_format'], $ct );
			$weather_arr['accutime']    = date_i18n( $wpf_vars['fc_time_format'], $ct );
			$iconfile                   = find_icon( $w['weathericon'] );
			$weather_arr['icon']        = 'icons/' . $iconfile;
			$weather_arr['iconcode']    = $w['weathericon'];
			$weather_arr['shorttext']   = wpf__( $w['weathericon'], $wpf_vars['wpf_language'] );
			$weather_arr['temperature'] = $w['temperature'] . '&deg;' . $w['un_temp'];
			$weather_arr['realfeel']    = $w['realfeel'] . '&deg;' . $w['un_temp'];
			// workaround different pressure values returned by accuweather.
			$press = round( (float) $w['pressure'], 0 );

			if ( strlen( $press ) == 3 && substr( $press, 0, 1 ) == '1' ) {
				$press = $press * 10;
			}
			$weather_arr['pressure']     = $press . ' ' . $w['un_pres'];
			$weather_arr['humidity']     = round( (float) $w['humidity'], 0 );
			$weather_arr['windspeed']    = windstr( $wpf_vars['metric'], $w['windspeed'], $wpf_vars['windunit'] );
			$weather_arr['winddir']      = translate_winddir( $w['winddirection'], 'wp-forecast_' . $wpf_vars['wpf_language'] );
			$weather_arr['winddir_orig'] = str_replace( 'O', 'E', $w['winddirection'] );
			$weather_arr['windgusts']    = windstr( $wpf_vars['metric'], $w['wgusts'], $wpf_vars['windunit'] );
			$sunarr                      = explode( ' ', $w['sun'] );
			$weather_arr['sunrise']      = gmdate( $wpf_vars['fc_time_format'], strtotime( $sunarr[0] ) );
			$weather_arr['sunset']       = gmdate( $wpf_vars['fc_time_format'], strtotime( $sunarr[1] ) );
			$weather_arr['copyright']    = '<a href="http://www.accuweather.com">&copy; ' . gmdate( 'Y' ) . ' AccuWeather, Inc.</a>';
			$weather_arr['uvindex']      = $w['uvindex'];
			// calc values for forecast.

			for ( $i = 1;$i < 10;$i++ ) {
				// daytime forecast.
				$weather_arr[ 'fc_obsdate_' . $i ]         = date_i18n( $wpf_vars['fc_date_format'], strtotime( $w[ 'fc_obsdate_' . $i ] ) );
				$iconfile                                  = find_icon( $w[ 'fc_dt_icon_' . $i ] );
				$weather_arr[ 'fc_dt_icon_' . $i ]         = 'icons/' . $iconfile;
				$weather_arr[ 'fc_dt_iconcode_' . $i ]     = $w[ 'fc_dt_icon_' . $i ];
				$weather_arr[ 'fc_dt_desc_' . $i ]         = wpf__( $w[ 'fc_dt_icon_' . $i ], $wpf_vars['wpf_language'] );
				$weather_arr[ 'fc_dt_htemp_' . $i ]        = $w[ 'fc_dt_htemp_' . $i ] . '&deg;' . $w['un_temp'];
				$wstr                                      = windstr( $wpf_vars['metric'], $w[ 'fc_dt_windspeed_' . $i ], $wpf_vars['windunit'] );
				$weather_arr[ 'fc_dt_windspeed_' . $i ]    = $wstr;
				$weather_arr[ 'fc_dt_winddir_' . $i ]      = translate_winddir( $w[ 'fc_dt_winddir_' . $i ], 'wp-forecast_' . $wpf_vars['wpf_language'] );
				$weather_arr[ 'fc_dt_winddir_orig_' . $i ] = str_replace( 'O', 'E', $w[ 'fc_dt_winddir_' . $i ] );
				$weather_arr[ 'fc_dt_wgusts_' . $i ]       = windstr( $wpf_vars['metric'], $w[ 'fc_dt_wgusts_' . $i ], $wpf_vars['windunit'] );
				$weather_arr[ 'fc_dt_maxuv_' . $i ]        = $w[ 'fc_dt_maxuv_' . $i ];
				// nighttime forecast.
				$iconfile                                  = find_icon( $w[ 'fc_nt_icon_' . $i ] );
				$weather_arr[ 'fc_nt_icon_' . $i ]         = 'icons/' . $iconfile;
				$weather_arr[ 'fc_nt_iconcode_' . $i ]     = $w[ 'fc_nt_icon_' . $i ];
				$weather_arr[ 'fc_nt_desc_' . $i ]         = wpf__( $w[ 'fc_nt_icon_' . $i ], $wpf_vars['wpf_language'] );
				$weather_arr[ 'fc_nt_ltemp_' . $i ]        = $w[ 'fc_nt_ltemp_' . $i ] . '&deg;' . $w['un_temp'];
				$wstr                                      = windstr( $wpf_vars['metric'], $w[ 'fc_nt_windspeed_' . $i ], $wpf_vars['windunit'] );
				$weather_arr[ 'fc_nt_windspeed_' . $i ]    = $wstr;
				$weather_arr[ 'fc_nt_winddir_' . $i ]      = translate_winddir( $w[ 'fc_nt_winddir_' . $i ], 'wp-forecast_' . $wpf_vars['wpf_language'] );
				$weather_arr[ 'fc_nt_winddir_orig_' . $i ] = str_replace( 'O', 'E', $w[ 'fc_nt_winddir_' . $i ] );
				$weather_arr[ 'fc_nt_wgusts_' . $i ]       = windstr( $wpf_vars['metric'], $w[ 'fc_nt_wgusts_' . $i ], $wpf_vars['windunit'] );
				$weather_arr[ 'fc_nt_maxuv_' . $i ]        = $w[ 'fc_nt_maxuv_' . $i ];
				// additional info.
				$weather_arr['lat'] = $w['lat'];
				$weather_arr['lon'] = $w['lon'];
			}
		}
		// fill failure anyway.
		$weather_arr['failure'] = ( isset( $w['failure'] ) ? $w['failure'] : '' );
		return $weather_arr;
	}
} // end of function exists.
